@@ -545,7 +545,7 @@ fatal("! Cannot open input file ",web_file_name);
 strcpy(web_file_name,found_filename);
 free(found_filename);
 #if defined(MIKTEX)
-MiKTeX::Core::PathName sourceDir(web_file_name);
+MiKTeX::Util::PathName sourceDir(web_file_name);
 sourceDir.RemoveFileSpec();
 MiKTeX::Core::Session::Get()->AddInputDirectory(sourceDir,true);
 #endif
