@@ -45,7 +45,9 @@
 #endif
 
 #include <thread>
+#include <tuple>
 
+#include <miktex/Core/AutoResource>
 #include <miktex/Core/Directory>
 #include <miktex/Core/Environment>
 #include <miktex/Core/CommandLineBuilder>
@@ -192,6 +194,29 @@ private:
   int twofd[2] = { -1, -1 };
 };
 
+tuple<char*, char**> CreateEnvironmentBlock(const unordered_map<string, string>& envMap)
+{
+  size_t envSize = 0;
+  for (const auto& p : envMap)
+  {
+    envSize += p.first.length() + 1 + p.second.length() + 1;
+  }
+  char* environmentStrings = new char[envSize];
+  char** environmentPointers = new char*[envMap.size() + 1];
+  size_t stringIdx = 0;
+  size_t pointerIdx = 0;
+  for (const auto& p : envMap)
+  {
+    string s = fmt::format("{}={}", p.first, p.second);
+    strcpy(environmentStrings + stringIdx, s.c_str());
+    environmentPointers[pointerIdx] = environmentStrings + stringIdx;
+    stringIdx += s.length() + 1;
+    ++pointerIdx;
+  }
+  environmentPointers[pointerIdx] = nullptr;
+  return make_tuple(environmentStrings, environmentPointers);
+}
+
 unique_ptr<Process> Process::Start(const ProcessStartInfo& startinfo)
 {
   return make_unique<unxProcess>(startinfo);
@@ -264,7 +289,6 @@ void unxProcess::Create()
   }
 
   tmpFile = TemporaryFile::Create();
-  tmpEnv.Set(MIKTEX_ENV_EXCEPTION_PATH, tmpFile->GetPathName().ToString());
 
   // fork
   if (session != nullptr)
@@ -318,14 +342,6 @@ void unxProcess::Create()
       pipeStdout.Dispose();
       pipeStderr.Dispose();
       pipeStdin.Dispose();
-      if (session != nullptr)
-      {
-        session->SetEnvironmentVariables();
-      }
-      if (!startinfo.WorkingDirectory.empty())
-      {
-        Directory::SetCurrent(PathName(startinfo.WorkingDirectory));
-      }
       if (startinfo.Daemonize)
       {
         if (setsid() == -1)
@@ -345,9 +361,28 @@ void unxProcess::Create()
           }
         }
         session->trace_process->WriteLine("core", TraceLevel::Info, fmt::format("execv: \"{0}\", [ {1} ]", fileName, args));
+        unordered_map<string, string> envMap = session->CreateChildEnvironment(!startinfo.WorkingDirectory.empty());
+        envMap[MIKTEX_ENV_EXCEPTION_PATH] = tmpFile->GetPathName().ToString();
+        char* environmentStrings;
+        char** environmentPointers;
+        tie(environmentStrings, environmentPointers) = CreateEnvironmentBlock(envMap);
+        MIKTEX_AUTO(delete[]environmentStrings);
+        MIKTEX_AUTO(delete[]environmentPointers);
+        if (!startinfo.WorkingDirectory.empty())
+        {
+          Directory::SetCurrent(PathName(startinfo.WorkingDirectory));
+        }
+        execve(fileName.GetData(), const_cast<char*const*>(argv.GetArgv()), const_cast<char*const*>(environmentPointers));
       }
-      execv(fileName.GetData(), const_cast<char*const*>(argv.GetArgv()));
-      perror("execv failed");
+      else
+      {
+        if (!startinfo.WorkingDirectory.empty())
+        {
+          Directory::SetCurrent(PathName(startinfo.WorkingDirectory));
+        }
+        execv(fileName.GetData(), const_cast<char*const*>(argv.GetArgv()));
+      }
+      perror("execve failed");
     }
     catch (const exception&)
     {
@@ -419,7 +454,6 @@ void unxProcess::Close()
   this->pid = -1;
   if (tmpFile != nullptr)
   {
-    tmpEnv.Restore();
     tmpFile->Delete();
     tmpFile = nullptr;
   }
@@ -538,7 +572,6 @@ ProcessExitStatus unxProcess::get_ExitStatus() const
   }
 }
 
-
 int unxProcess::get_ExitCode() const
 {
   if (WIFEXITED(status) != 0)
@@ -835,7 +868,6 @@ ProcessInfo unxProcess::GetProcessInfo()
   return processInfo;
 }
 
-
 void Process::Overlay(const PathName& fileName, const vector<string>& arguments)
 {
   MIKTEX_EXPECT(!fileName.Empty());
@@ -846,15 +878,23 @@ void Process::Overlay(const PathName& fileName, const vector<string>& arguments)
   if (session != nullptr)
   {
     session->UnloadFilenameDatabase();
-    session->SetEnvironmentVariables();
-    session->trace_process->WriteLine("core", TraceLevel::Info, fmt::format("execv: {0}", fileName));
+    session->trace_process->WriteLine("core", TraceLevel::Info, fmt::format("execve: {0}", fileName));
     for (int idx = 0; argv[idx] != nullptr; ++idx)
     {
       session->trace_process->WriteLine("core", TraceLevel::Info, fmt::format(" argv[{0}]: {1}", idx, argv[idx]));
     }
+    unordered_map<string, string> envMap = session->CreateChildEnvironment(false);
+    char* environmentStrings;
+    char** environmentPointers;
+    tie(environmentStrings, environmentPointers) = CreateEnvironmentBlock(envMap);
+    MIKTEX_AUTO(delete[]environmentStrings);
+    MIKTEX_AUTO(delete[]environmentPointers);
+    execve(fileName.GetData(), const_cast<char*const*>(argv.GetArgv()), const_cast<char*const*>(environmentPointers));
+  }
+  else
+  {
+    execv(fileName.GetData(), const_cast<char*const*>(argv.GetArgv()));
   }
-
-  execv(fileName.GetData(), const_cast<char*const*>(argv.GetArgv()));
 
   MIKTEX_UNEXPECTED();
 }
