@@ -47,6 +47,10 @@
 #include <miktex/Core/equal_icase>
 #include <miktex/Core/hash_icase>
 
+#if defined(MIKTEX_WINDOWS)
+#include <miktex/Core/win/COMInitializer>
+#endif
+
 #include "Fndb/FileNameDatabase.h"
 #include "RootDirectoryInternals.h"
 
@@ -1177,7 +1181,8 @@ private:
 
 #if defined(MIKTEX_WINDOWS) && USE_LOCAL_SERVER
 private:
-  struct LocalServer
+  struct LocalServer :
+    public MiKTeX::Core::COMInitializer
   {
     ATL::CComQIPtr<MiKTeXSessionLib::ISession> pSession;
   };
