@@ -1,6 +1,6 @@
 /* filetypes.cpp: MiKTeX file type registry
 
-   Copyright (C) 1996-2020 Christian Schenk
+   Copyright (C) 1996-2021 Christian Schenk
 
    This file is part of the MiKTeX Core Library.
 
@@ -21,7 +21,7 @@
 
 #include "config.h"
 
-#include <miktex/Core/ConfigNames>
+#include <miktex/Configuration/ConfigNames>
 #include <miktex/Core/Environment>
 #include <miktex/Core/Paths>
 
@@ -31,6 +31,7 @@
 
 using namespace std;
 
+using namespace MiKTeX::Configuration;
 using namespace MiKTeX::Core;
 using namespace MiKTeX::Util;
 
@@ -136,7 +137,7 @@ void SessionImpl::RegisterFileType(FileType fileType)
 #elif defined(MIKTEX_EXE_FILE_SUFFIX)
     extensions.push_back(MIKTEX_EXE_FILE_SUFFIX);
 #endif
-    PathName localBinDir = GetSpecialPath(MiKTeX::Core::SpecialPath::LinkTargetDirectory);
+    PathName localBinDir = GetSpecialPath(SpecialPath::LinkTargetDirectory);
     localBinDir.Canonicalize();
     if (std::find(searchPath.begin(), searchPath.end(), localBinDir.ToString()) == searchPath.end())
     {
