@@ -1,6 +1,6 @@
 /* winSession.cpp: Windows specials
 
-   Copyright (C) 1996-2020 Christian Schenk
+   Copyright (C) 1996-2021 Christian Schenk
 
    This file is part of the MiKTeX Core Library.
 
@@ -28,7 +28,8 @@
 #include <htmlhelp.h>
 #include <shlobj.h>
 
-#include <miktex/Core/ConfigNames>
+#include <miktex/Configuration/ConfigNames>
+#include <miktex/Core/BufferSizes>
 #include <miktex/Core/Directory>
 #include <miktex/Core/Paths>
 #include <miktex/Core/win/HResult>
@@ -42,34 +43,19 @@
 
 using namespace std;
 
+using namespace MiKTeX::Configuration;
 using namespace MiKTeX::Core;
 using namespace MiKTeX::Util;
 
-void SessionImpl::MyCoInitialize()
-{
-  HResult hr = CoInitializeEx(nullptr, COINIT_MULTITHREADED);
-  if (hr.Failed())
-  {
-    MIKTEX_FATAL_ERROR_2(T_("The COM library could not be initialized."), "hr", hr.GetText());
-  }
-  ++numCoInitialize;
-}
-
-void SessionImpl::MyCoUninitialize()
-{
-  if (numCoInitialize == 0)
-  {
-    MIKTEX_UNEXPECTED();
-  }
-  CoUninitialize();
-  --numCoInitialize;
-}
-
 #if USE_LOCAL_SERVER
 void SessionImpl::ConnectToServer()
 {
   const char* MSG_CANNOT_START_SERVER = T_("Cannot start MiKTeX session.");
-  if (localServer.pSession == nullptr)
+  if (localServer == nullptr)
+  {
+    localServer = make_unique<LocalServer>();
+  }
+  if (localServer->pSession == nullptr)
   {
     WCHAR wszCLSID[50];
     if (StringFromGUID2(__uuidof(MiKTeXSessionLib::MAKE_CURVER_ID(MiKTeXSession)), wszCLSID, sizeof(wszCLSID) / sizeof(wszCLSID[0])) < 0)
@@ -84,15 +70,10 @@ void SessionImpl::ConnectToServer()
     bo.cbStruct = sizeof(bo);
     bo.hwnd = GetForegroundWindow();
     bo.dwClassContext = CLSCTX_LOCAL_SERVER;
-    HResult hr = CoGetObject(monikerName.c_str(), &bo, __uuidof(MiKTeXSessionLib::ISession), reinterpret_cast<void**>(&localServer.pSession));
-    if (hr == CO_E_NOTINITIALIZED)
-    {
-      MyCoInitialize();
-      hr = CoGetObject(monikerName.c_str(), &bo, __uuidof(MiKTeXSessionLib::ISession), reinterpret_cast<void**>(&localServer.pSession));
-    }
+    HResult hr = CoGetObject(monikerName.c_str(), &bo, __uuidof(MiKTeXSessionLib::ISession), reinterpret_cast<void**>(&localServer->pSession));
     if (hr.Failed())
     {
-      MIKTEX_FATAL_ERROR_2(MSG_CANNOT_START_SERVER, "hr", hr.GetText());
+      MIKTEX_FATAL_ERROR_2(MSG_CANNOT_START_SERVER, "hr", hr.ToString());
     }
   }
 }
@@ -611,7 +592,8 @@ bool SessionImpl::GetAcrobatFontDir(PathName& path)
 
     RemoveDirectoryDelimiter(fontDir.GetData());
 
-    acrobatFontDir = GetFullyQualifiedPath(fontDir.GetData());
+    acrobatFontDir = fontDir;
+    acrobatFontDir.Convert({ ConvertPathNameOption::MakeFullyQualified });
   }
 
   if (acrobatFontDir.GetLength() == 0)
@@ -648,7 +630,8 @@ bool SessionImpl::GetATMFontDir(PathName& path)
 
     RemoveDirectoryDelimiter(fontDir.GetData());
 
-    atmFontDir = GetFullyQualifiedPath(fontDir.GetData());
+    atmFontDir = fontDir;
+    atmFontDir.Convert({ ConvertPathNameOption::MakeFullyQualified });
   }
 
   if (atmFontDir.GetLength() == 0)
