@@ -1,6 +1,6 @@
 /* unxUtil.cpp: 
 
-   Copyright (C) 1996-2020 Christian Schenk
+   Copyright (C) 1996-2021 Christian Schenk
 
    This file is part of the MiKTeX Core Library.
 
@@ -29,7 +29,7 @@
 #include <fmt/ostream.h>
 
 #include <miktex/Core/Directory>
-#include <miktex/Core/PathName>
+#include <miktex/Util/PathName>
 
 #include "internal.h"
 
@@ -37,7 +37,9 @@
 
 using namespace std;
 
+using namespace MiKTeX::Configuration;
 using namespace MiKTeX::Core;
+using namespace MiKTeX::Util;
 
 string Utils::GetOSVersionString()
 {
@@ -97,20 +99,6 @@ void Utils::CheckHeap()
 {
 }
 
-void Utils::CanonicalizePathName(PathName& path)
-{
-  char resolved[PATH_MAX + 1];
-  if (realpath(path.GetData(), resolved) == nullptr)
-  {
-    if (errno == ENOENT)
-    {
-      return;
-    }
-    MIKTEX_FATAL_CRT_ERROR_2("realpath", "path", path.ToString());
-  }
-  path = resolved;
-}
-
 void Utils::ShowWebPage(const string& url)
 {
   UNIMPLEMENTED();
