@@ -37,6 +37,10 @@
 #include "PackageManagerImpl.h"
 #include "RepositoryManifest.h"
 
+#if defined(MIKTEX_WINDOWS)
+#include <miktex/Core/win/COMInitializer>
+#endif
+
 #if defined(MIKTEX_WINDOWS) && USE_LOCAL_SERVER
 #include "COM/com-internal.h"
 #endif
@@ -456,7 +460,8 @@ private:
 
 #if defined(MIKTEX_WINDOWS) && USE_LOCAL_SERVER
 private:
-  struct LocalServer
+  struct LocalServer :
+    public MiKTeX::Core::COMInitializer
   {
     ATL::CComQIPtr<MiKTeXPackageManagerLib::IPackageManager> pManager;
     ATL::CComPtr<MiKTeXPackageManagerLib::IPackageInstaller> pInstaller;
