@@ -1,6 +1,6 @@
 /* InstallPackageDialog.cpp:
 
-   Copyright (C) 2008-2020 Christian Schenk
+   Copyright (C) 2008-2021 Christian Schenk
 
    This file is part of the MiKTeX UI Library.
 
@@ -42,10 +42,13 @@
 
 #include "InstallPackageDialog.h"
 
+using namespace std;
+
+using namespace MiKTeX::Configuration;
 using namespace MiKTeX::Core;
 using namespace MiKTeX::Packages;
 using namespace MiKTeX::UI::Qt;
-using namespace std;
+using namespace MiKTeX::Util;
 
 InstallPackageDialog::InstallPackageDialog(QWidget* parent, shared_ptr<PackageManager> packageManager, const string& packageName, const string& trigger) :
   QDialog(parent),
@@ -60,9 +63,9 @@ InstallPackageDialog::InstallPackageDialog(QWidget* parent, shared_ptr<PackageMa
     {
       MIKTEX_UNEXPECTED();
     }
-    okayButton->setText(T_("Install"));
-    lblPackageName->setText(QString::fromLocal8Bit(packageName.c_str()));
-    lblMissingFile->setText(QString::fromLocal8Bit(trigger.c_str()));
+    okayButton->setText(tr("Install"));
+    lblPackageName->setText(QString::fromStdString(packageName));
+    lblMissingFile->setText(QString::fromStdString(PathName(trigger).GetFileName().ToDisplayString()));
     PackageInfo packageInfo = packageManager->GetPackageInfo(packageName);
     string repository;
     RepositoryType repositoryType(RepositoryType::Unknown);
@@ -72,7 +75,7 @@ InstallPackageDialog::InstallPackageDialog(QWidget* parent, shared_ptr<PackageMa
     }
     else
     {
-      leInstallationSource->setText(T_("<Random package repository>"));
+      leInstallationSource->setText(tr("<Random package repository>"));
     }
     PathName commonInstallRoot = session->GetSpecialPath(SpecialPath::CommonInstallRoot);
     PathName userInstallRoot = session->IsAdminMode() ? PathName() : session->GetSpecialPath(SpecialPath::UserInstallRoot);
@@ -80,7 +83,7 @@ InstallPackageDialog::InstallPackageDialog(QWidget* parent, shared_ptr<PackageMa
     enableCommonInstall = enableCommonInstall && Directory::Exists(commonInstallRoot);
     if (enableCommonInstall)
     {
-      cbInstallationDirectory->addItem(T_("Anyone who uses this computer (all users)"), true);
+      cbInstallationDirectory->addItem(tr("<All users>"), true);
     }
     if (!session->IsAdminMode())
     {
@@ -94,7 +97,7 @@ InstallPackageDialog::InstallPackageDialog(QWidget* parent, shared_ptr<PackageMa
       }
       else if (GetLastError() == ERROR_NOT_LOGGED_ON)
       {
-        currentUser = T_("Unknown user");
+        currentUser = tr("<Unknown user>");
       }
       else
       {
@@ -109,7 +112,7 @@ InstallPackageDialog::InstallPackageDialog(QWidget* parent, shared_ptr<PackageMa
         currentUser += ")";
       }
 #else
-      currentUser = T_("The current user");
+      currentUser = tr("<Current user>");
 #endif
       cbInstallationDirectory->addItem(currentUser, false);
     }
