@@ -81,32 +81,17 @@ truedpi (const char *ident, double point_size, unsigned bdpi)
 }
 
 static FILE *
-dpx_open_pk_font_at (const char *ident, unsigned dpi, char **pkname)
+dpx_open_pk_font_at (const char *ident, unsigned dpi)
 {
   FILE  *fp;
   char  *fqpn;
   kpse_glyph_file_type kpse_file_info;
-/*
- * ident can be such as "cmr10.pfb":
- * https://tug.org/pipermail/dvipdfmx/2020-December/000142.html.
- * As a workaround, we drop a suffix if it exists.
- * 
- */
-  char  *ident_nosuffix;
-  ident_nosuffix = xstrdup(ident);
-  fqpn = strrchr(ident_nosuffix, '.');
-  if (fqpn) {
-    *fqpn = '\0';
-  }
-  fqpn = kpse_find_glyph(ident_nosuffix, dpi, kpse_pk_format, &kpse_file_info);
 
+  fqpn = kpse_find_glyph(ident, dpi, kpse_pk_format, &kpse_file_info);
   if (!fqpn)
     return  NULL;
   fp   = MFOPEN(fqpn, FOPEN_RBIN_MODE);
   RELEASE(fqpn);
-  *pkname = NEW(strlen(ident_nosuffix)+12, char);
-  (void)snprintf(*pkname, strlen(ident_nosuffix)+12,  "%s.%dpk", ident_nosuffix, dpi);
-  RELEASE(ident_nosuffix);
 
   return  fp;
 }
@@ -117,7 +102,6 @@ pdf_font_open_pkfont (pdf_font *font, const char *ident, int index, int encoding
 {
   unsigned  dpi;
   FILE     *fp;
-  char     *pkname;
 
   if (!ident || point_size <= 0.0)
     return  -1;
@@ -129,7 +113,7 @@ pdf_font_open_pkfont (pdf_font *font, const char *ident, int index, int encoding
   }
 
   dpi = truedpi(ident, point_size, base_dpi);
-  fp  = dpx_open_pk_font_at(ident, dpi, &pkname);
+  fp  = dpx_open_pk_font_at(ident, dpi);
   if (!fp)
     return  -1;
   MFCLOSE(fp);
@@ -137,7 +121,8 @@ pdf_font_open_pkfont (pdf_font *font, const char *ident, int index, int encoding
   /* Type 3 fonts doesn't have FontName.
    * FontFamily is recommended for PDF 1.5.
    */
-  font->fontname = pkname;
+  font->fontname = NEW(strlen(ident)+1, char);
+  strcpy(font->fontname, ident);
 
   if (encoding_id >= 0) {
     pdf_encoding_used_by_type3(encoding_id);
@@ -500,7 +485,6 @@ create_pk_CharProc_stream (struct pk_header_ *pkh,
 int
 pdf_font_load_pkfont (pdf_font *font)
 {
-  char     *pkname;
   pdf_obj  *fontdict;
   char     *usedchars;
   char     *ident;
@@ -536,11 +520,11 @@ pdf_font_load_pkfont (pdf_font *font)
   ASSERT(ident && usedchars && point_size > 0.0);
 
   dpi  = truedpi(ident, point_size, base_dpi);
-  fp   = dpx_open_pk_font_at(ident, dpi, &pkname);
+  fp   = dpx_open_pk_font_at(ident, dpi);
   if (!fp) {
     ERROR("Could not find/open PK font file: %s (at %udpi)", ident, dpi);
   }
-  font->filename = pkname;
+
   memset(charavail, 0, 256);
   charprocs  = pdf_new_dict();
   /* Include bitmap as 72dpi image:
