@@ -1,6 +1,6 @@
 /*
 	This is part of TeXworks, an environment for working with TeX documents
-	Copyright (C) 2007-2020  Jonathan Kew, Stefan Löffler, Charlie Sharpsteen
+	Copyright (C) 2007-2021  Jonathan Kew, Stefan Löffler, Charlie Sharpsteen
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -24,50 +24,45 @@
 #include <miktex/utf8wrap.h>
 #endif
 #include "TWApp.h"
-#include "TWUtils.h"
-#include "TeXDocumentWindow.h"
-#include "PDFDocumentWindow.h"
-#include "PrefsDialog.h"
-#include "DefaultPrefs.h"
-#include "TemplateDialog.h"
-#include "TWSystemCmd.h"
-#include "Settings.h"
-#include "document/SpellChecker.h"
-#include "scripting/ScriptAPI.h"
-
-#include "TWVersion.h"
-#include "ResourcesDialog.h"
-#include "TWTextCodecs.h"
 
 #if !defined(MIKTEX)
-#if defined(Q_OS_WIN)
-#include "DefaultBinaryPathsWin.h"
-#else
 #include "DefaultBinaryPaths.h"
 #endif
-#endif
+#include "DefaultPrefs.h"
+#include "PDFDocumentWindow.h"
+#include "PrefsDialog.h"
+#include "ResourcesDialog.h"
+#include "Settings.h"
+#include "TWUtils.h"
+#include "TeXDocumentWindow.h"
+#include "TemplateDialog.h"
+#include "document/SpellChecker.h"
+#include "scripting/ScriptAPI.h"
+#include "utils/ResourcesLibrary.h"
+#include "utils/SystemCommand.h"
+#include "utils/TextCodecs.h"
+#include "utils/VersionInfo.h"
 
-#include <QMessageBox>
-#include <QFileDialog>
-#include <QString>
-#include <QMenuBar>
-#include <QMenu>
 #include <QAction>
-#include <QSettings>
-#include <QStringList>
+#include <QDesktopServices>
 #include <QEvent>
+#include <QFileDialog>
 #include <QKeyEvent>
 #include <QKeySequence>
-#include <QDesktopWidget>
-#include <QTextCodec>
+#include <QLibraryInfo>
 #include <QLocale>
+#include <QMenu>
+#include <QMenuBar>
+#include <QMessageBox>
+#include <QSettings>
+#include <QString>
+#include <QStringList>
+#include <QTextCodec>
 #include <QTranslator>
 #include <QUrl>
-#include <QDesktopServices>
-#include <QLibraryInfo>
 
 #if defined(Q_OS_DARWIN)
-#include <CoreServices/CoreServices.h>
+extern QString GetMacOSVersionString();
 #endif
 
 #if defined(Q_OS_WIN)
@@ -85,6 +80,40 @@ TWApp *TWApp::theAppInstance = nullptr;
 
 const QEvent::Type TWDocumentOpenEvent::type = static_cast<QEvent::Type>(QEvent::registerEventType());
 
+QString replaceEnvironmentVariables(const QString & s)
+{
+	QString rv{s};
+
+	// If there is nothing to replace, don't bother trying
+#ifdef Q_OS_WINDOWS
+	if (!s.contains(QStringLiteral("%"))) {
+		return rv;
+	}
+#else
+	if (!s.contains(QStringLiteral("$"))) {
+		return rv;
+	}
+#endif
+
+	QProcessEnvironment env{QProcessEnvironment::systemEnvironment()};
+	QStringList vars = env.keys();
+	// Sort the variable names from longest to shortest to appropriately handle
+	// cases like $HOMEPATH (if $HOME also exists)
+	std::sort(vars.begin(), vars.end(), [](const QString & a, const QString & b) { return a.length() > b.length(); });
+
+	foreach(const QString & var, vars) {
+#ifdef Q_OS_WINDOWS
+		// Replace "%VAR%" by the value of the corresponding environment variable
+		rv = rv.replace(QStringLiteral("%") + var + QStringLiteral("%"), env.value(var), Qt::CaseInsensitive);
+#else
+		// Replace "${VAR}" and "$VAR" by the value of the corresponding
+		// environment variable (but not "\$HOME")
+		QRegularExpression re{QStringLiteral("(?<!\\\\)\\$(%1|\\{%1\\})").arg(QRegularExpression::escape(var))};
+		rv.replace(re, env.value(var));
+#endif
+	}
+	return rv;
+}
 
 TWApp::TWApp(int &argc, char **argv)
 	: QApplication(argc, argv)
@@ -109,8 +138,13 @@ TWApp::~TWApp()
 
 void TWApp::init()
 {
-	customTextCodecs << new MacCentralEurRomanCodec();
+#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
+	constexpr auto SkipEmptyParts = QString::SkipEmptyParts;
+#else
+	constexpr auto SkipEmptyParts = Qt::SkipEmptyParts;
+#endif
 
+	QIcon::setThemeName(QStringLiteral("tango-texworks"));
 	QIcon appIcon;
 #if defined(Q_OS_UNIX) && !defined(Q_OS_DARWIN)
 	// The Compiz window manager doesn't seem to support icons larger than
@@ -119,11 +153,11 @@ void TWApp::init()
 #endif
 	appIcon.addFile(QString::fromLatin1(":/images/images/TeXworks.png"));
 	setWindowIcon(appIcon);
-	
+
 	setOrganizationName(QString::fromLatin1("TUG"));
 	setOrganizationDomain(QString::fromLatin1("tug.org"));
 	setApplicationName(QString::fromLatin1(TEXWORKS_NAME));
-	
+
 	// <Check for portable mode>
 #if defined(Q_OS_DARWIN)
 	QDir appDir(applicationDirPath() + QLatin1String("/../../..")); // move up to dir containing the .app package
@@ -142,12 +176,12 @@ void TWApp::init()
 		}
 		if (portable.contains(QString::fromLatin1("libpath"))) {
 			if (libPath.cd(portable.value(QString::fromLatin1("libpath")).toString())) {
-				portableLibPath = libPath.absolutePath();
+				Tw::Utils::ResourcesLibrary::setPortableLibPath(libPath.absolutePath());
 			}
 		}
 		if (portable.contains(QString::fromLatin1("defaultbinpaths"))) {
 			defaultBinPaths = new QStringList;
-			*defaultBinPaths = portable.value(QString::fromLatin1("defaultbinpaths")).toString().split(QString::fromLatin1(PATH_LIST_SEP), QString::SkipEmptyParts);
+			*defaultBinPaths = portable.value(QString::fromLatin1("defaultbinpaths")).toString().split(QString::fromLatin1(PATH_LIST_SEP), SkipEmptyParts);
 		}
 	}
 #if defined(MIKTEX_WINDOWS)
@@ -180,7 +214,7 @@ void TWApp::init()
 	envPath = QString::fromLocal8Bit(getenv("TW_LIBPATH"));
 #endif
 	if (!envPath.isNull() && libPath.cd(envPath)) {
-		portableLibPath = libPath.absolutePath();
+		Tw::Utils::ResourcesLibrary::setPortableLibPath(libPath.absolutePath());
 	}
 	// </Check for portable mode>
 
@@ -188,7 +222,7 @@ void TWApp::init()
 	theAppInstance = this;
 
 	Tw::Settings settings;
-	
+
 	QString locale = settings.value(QString::fromLatin1("locale"), QLocale::system().name()).toString();
 	applyTranslation(locale);
 
@@ -212,56 +246,42 @@ void TWApp::init()
 	menuFile = menuBar->addMenu(tr("File"));
 
 	actionNew = new QAction(tr("New"), this);
-	actionNew->setIcon(QIcon(QString::fromLatin1(":/images/tango/document-new.png")));
+	actionNew->setIcon(QIcon::fromTheme(QStringLiteral("document-new")));
 	menuFile->addAction(actionNew);
-	connect(actionNew, SIGNAL(triggered()), this, SLOT(newFile()));
+	connect(actionNew, &QAction::triggered, this, &TWApp::newFile);
 
 	actionNew_from_Template = new QAction(tr("New from Template..."), this);
 	menuFile->addAction(actionNew_from_Template);
-	connect(actionNew_from_Template, SIGNAL(triggered()), this, SLOT(newFromTemplate()));
-
-	actionPreferences = new QAction(tr("Preferences..."), this);
-	actionPreferences->setIcon(QIcon(QString::fromLatin1(":/images/tango/preferences-system.png")));
-	actionPreferences->setMenuRole(QAction::PreferencesRole);
-	menuFile->addAction(actionPreferences);
-	connect(actionPreferences, SIGNAL(triggered()), this, SLOT(preferences()));
+	connect(actionNew_from_Template, &QAction::triggered, this, &TWApp::newFromTemplate);
 
 	actionOpen = new QAction(tr("Open..."), this);
-	actionOpen->setIcon(QIcon(QString::fromLatin1(":/images/tango/document-open.png")));
+	actionOpen->setIcon(QIcon::fromTheme(QStringLiteral("document-open")));
 	menuFile->addAction(actionOpen);
-	connect(actionOpen, SIGNAL(triggered()), this, SLOT(open()));
+	connect(actionOpen, &QAction::triggered, [=]() { this->open(); });
 
 	menuRecent = new QMenu(tr("Open Recent"));
 	actionClear_Recent_Files = menuRecent->addAction(tr("Clear Recent Files"));
 	actionClear_Recent_Files->setEnabled(false);
-	connect(actionClear_Recent_Files, SIGNAL(triggered()), this, SLOT(clearRecentFiles()));
+	connect(actionClear_Recent_Files, &QAction::triggered, this, &TWApp::clearRecentFiles);
 	updateRecentFileActions();
 	menuFile->addMenu(menuRecent);
 
-	actionQuit = new QAction(tr("Quit TeXworks"), this);
-	actionQuit->setMenuRole(QAction::QuitRole);
-	menuFile->addAction(actionQuit);
-	connect(actionQuit, SIGNAL(triggered()), this, SLOT(quit()));
-
 	menuHelp = menuBar->addMenu(tr("Help"));
 
 	homePageAction = new QAction(tr("Go to TeXworks home page"), this);
 	menuHelp->addAction(homePageAction);
-	connect(homePageAction, SIGNAL(triggered()), this, SLOT(goToHomePage()));
+	connect(homePageAction, &QAction::triggered, this, &TWApp::goToHomePage);
 	mailingListAction = new QAction(tr("Email to the mailing list"), this);
 	menuHelp->addAction(mailingListAction);
-	connect(mailingListAction, SIGNAL(triggered()), this, SLOT(writeToMailingList()));
+	connect(mailingListAction, &QAction::triggered, this, &TWApp::writeToMailingList);
 	QAction* sep = new QAction(this);
 	sep->setSeparator(true);
 	menuHelp->addAction(sep);
-	aboutAction = new QAction(tr("About " TEXWORKS_NAME "..."), this);
-	aboutAction->setMenuRole(QAction::AboutRole);
-	menuHelp->addAction(aboutAction);
-	connect(aboutAction, SIGNAL(triggered()), this, SLOT(about()));
-	
-	TWUtils::insertHelpMenuItems(menuHelp);
 
-	connect(this, SIGNAL(updatedTranslators()), this, SLOT(changeLanguage()));
+	TWUtils::insertHelpMenuItems(menuHelp);
+	recreateSpecialMenuItems();
+
+	connect(this, &TWApp::updatedTranslators, this, &TWApp::changeLanguage);
 	changeLanguage();
 #endif
 #if defined(MIKTEX)
@@ -279,9 +299,59 @@ void TWApp::maybeQuit()
 	closeAllWindows();
 #if defined(Q_OS_DARWIN)
 	setQuitOnLastWindowClosed(false);
+	// If maybeQuit() was called from the global menu (i.e., no windows were open),
+	// closeAllWindows() has no effect; so we have to check for this condition
+	// ourselves and quit Tw if necessary
+	bool isAnyWindowStillOpen = false;
+	for (QWidget * w : topLevelWidgets()) {
+		if (w && w->isWindow() && w->isVisible()) {
+			isAnyWindowStillOpen = true;
+			break;
+		}
+	}
+	if (!isAnyWindowStillOpen) {
+		quit();
+	}
 #endif
 }
 
+#if defined(Q_OS_DARWIN)
+
+void TWApp::recreateSpecialMenuItems()
+{
+	// This is an attempt to work around QTBUG-17941
+	// On macOS, certain special menu items (Quit, Preferences, About) are moved from
+	// the menus they are added to to the global system menu.
+	// If several menu items with the same role are created (e.g., each window creates
+	// a "Quit" item), only one (probably the last) such item is displayed in the
+	// system menu.
+	// When _any_ of those special actions is deleted (e.g. because their owning window is
+	// destroyed) - regardless of whether it was the current/only item with a given role -
+	// the corresponding system menu item vanishes.
+	// As a workaround, this function can re-create the global menu items to forcefully re-add
+	// the system menu items. This function has to be called _after any menu item with
+	// a special role has been deleted_ (e.g. by using QTimer::singleShot(0, ...)) and will
+	// override _all_ special menu items.
+
+	delete actionQuit;
+	actionQuit = menuFile->addAction(tr("Quit TeXworks"));
+	actionQuit->setMenuRole(QAction::QuitRole);
+	connect(actionQuit, &QAction::triggered, this, &TWApp::maybeQuit);
+
+	delete actionPreferences;
+	actionPreferences = menuFile->addAction(tr("Preferences..."));
+	actionPreferences->setIcon(QIcon::fromTheme(QStringLiteral("preferences-system")));
+	actionPreferences->setMenuRole(QAction::PreferencesRole);
+	connect(actionPreferences, &QAction::triggered, this, &TWApp::preferences);
+
+	delete aboutAction;
+	aboutAction = menuHelp->addAction(tr("About " TEXWORKS_NAME "..."));
+	aboutAction->setMenuRole(QAction::AboutRole);
+	connect(aboutAction, &QAction::triggered, this, &TWApp::about);
+}
+
+#endif // defined(Q_OS_DARWIN)
+
 void TWApp::changeLanguage()
 {
 #if defined(Q_OS_DARWIN)
@@ -309,14 +379,14 @@ void TWApp::about()
 {
 	QString aboutText = tr("<p>%1 is a simple environment for editing, typesetting, and previewing TeX documents.</p>").arg(QString::fromLatin1(TEXWORKS_NAME));
 	aboutText += QLatin1String("<small>");
-	aboutText += QLatin1String("<p>&#xA9; 2007-2020  Jonathan Kew, Stefan L&#xF6;ffler, Charlie Sharpsteen");
+	aboutText += QLatin1String("<p>&#xA9; 2007-2021  Jonathan Kew, Stefan L&#xF6;ffler, Charlie Sharpsteen");
 #if defined(MIKTEX)
-	aboutText += tr("<br>Version %1 (%2) [r.%3, %4]").arg(QString::fromLatin1(TEXWORKS_VERSION), QString::fromUtf8(MiKTeX::Core::Utils::GetMiKTeXBannerString().c_str()), TWUtils::gitCommitHash(), TWUtils::gitCommitDate().toLocalTime().toString(Qt::SystemLocaleShortDate));
+	aboutText += tr("<br>Version %1 (%2) [r.%3, %4]").arg(Tw::Utils::VersionInfo::versionString(), QString::fromUtf8(MiKTeX::Core::Utils::GetMiKTeXBannerString().c_str()), Tw::Utils::VersionInfo::gitCommitHash(), QLocale::system().toString(Tw::Utils::VersionInfo::gitCommitDate().toLocalTime(), QLocale::ShortFormat));
 #else
-	if (TWUtils::isGitInfoAvailable())
-		aboutText += tr("<br>Version %1 (%2) [r.%3, %4]").arg(QString::fromLatin1(TEXWORKS_VERSION), QString::fromLatin1(TW_BUILD_ID_STR), TWUtils::gitCommitHash(), TWUtils::gitCommitDate().toLocalTime().toString(Qt::SystemLocaleShortDate));
+	if (Tw::Utils::VersionInfo::isGitInfoAvailable())
+		aboutText += tr("<br>Version %1 (%2) [r.%3, %4]").arg(Tw::Utils::VersionInfo::versionString(), Tw::Utils::VersionInfo::buildIdString(), Tw::Utils::VersionInfo::gitCommitHash(), QLocale::system().toString(Tw::Utils::VersionInfo::gitCommitDate().toLocalTime(), QLocale::ShortFormat));
 	else
-		aboutText += tr("<br>Version %1 (%2)").arg(QString::fromLatin1(TEXWORKS_VERSION), QString::fromLatin1(TW_BUILD_ID_STR));
+		aboutText += tr("<br>Version %1 (%2)").arg(Tw::Utils::VersionInfo::versionString(), Tw::Utils::VersionInfo::buildIdString());
 #endif
 	aboutText += tr("<p>Distributed under the <a href=\"http://www.gnu.org/licenses/gpl-2.0.html\">GNU General Public License</a>, version 2 or (at your option) any later version.");
 	aboutText += tr("<p><a href=\"http://www.qt.io/\">Qt application framework</a> v%1 by The Qt Company.").arg(QString::fromLatin1(qVersion()));
@@ -357,21 +427,21 @@ QString TWApp::GetWindowsVersionString()
 	PGNSI pGNSI;
 	BOOL bOsVersionInfoEx;
 	QString result = QLatin1String("(unknown version)");
-	
+
 	memset(&si, 0, sizeof(SYSTEM_INFO));
 	memset(&osvi, 0, sizeof(OSVERSIONINFOEXA));
-	
+
 	osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEXA);
-	if ( !(bOsVersionInfoEx = GetVersionExA ((OSVERSIONINFOA *) &osvi)) )
+	if ( !(bOsVersionInfoEx = GetVersionExA (reinterpret_cast<OSVERSIONINFOA *>(&osvi))) )
 		return result;
-	
+
 	// Call GetNativeSystemInfo if supported or GetSystemInfo otherwise.
-	pGNSI = (PGNSI) GetProcAddress(GetModuleHandle(TEXT("kernel32.dll")), "GetNativeSystemInfo");
+	pGNSI = reinterpret_cast<PGNSI>(GetProcAddress(GetModuleHandle(TEXT("kernel32.dll")), "GetNativeSystemInfo"));
 	if (pGNSI)
 		pGNSI(&si);
 	else
 		GetSystemInfo(&si);
-	
+
 	if ( VER_PLATFORM_WIN32_NT == osvi.dwPlatformId && osvi.dwMajorVersion > 4 ) {
 		if ( osvi.dwMajorVersion == 10 ) {
 			if ( osvi.dwMinorVersion == 0 ) {
@@ -433,7 +503,7 @@ QString TWApp::GetWindowsVersionString()
 		}
 		else if ( osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 0 ) {
 			result = QLatin1String("2000 ");
-			
+
 			if ( osvi.wProductType == VER_NT_WORKSTATION ) {
 				result += QLatin1String("Professional");
 			}
@@ -446,12 +516,12 @@ QString TWApp::GetWindowsVersionString()
 					result += QLatin1String("Server");
 			}
 		}
-		
+
 		if ( strlen(osvi.szCSDVersion) > 0 ) {
 			result += QChar::fromLatin1(' ');
 			result += QLatin1String(osvi.szCSDVersion);
 		}
-		
+
 		if ( osvi.dwMajorVersion >= 6 ) {
 			if ( si.wProcessorArchitecture==PROCESSOR_ARCHITECTURE_AMD64 )
 				result += QLatin1String(", 64-bit");
@@ -459,7 +529,7 @@ QString TWApp::GetWindowsVersionString()
 				result += QLatin1String(", 32-bit");
 		}
 	}
-	
+
 	return result;
 }
 #endif
@@ -467,36 +537,34 @@ QString TWApp::GetWindowsVersionString()
 unsigned int TWApp::GetWindowsVersion()
 {
 	OSVERSIONINFOEXA osvi;
-	
+
 	memset(&osvi, 0, sizeof(OSVERSIONINFOEXA));
-	
+
 	osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEXA);
-	if ( !GetVersionExA ((OSVERSIONINFOA *) &osvi) )
+	if (!GetVersionExA (reinterpret_cast<OSVERSIONINFOA *>(&osvi)))
 		return 0;
-	
+
 	return (osvi.dwMajorVersion << 24) | (osvi.dwMinorVersion << 16) | (osvi.wServicePackMajor << 8) | (osvi.wServicePackMinor << 0);
 }
 #endif
 
-const QStringList TWApp::getBinaryPaths(QStringList& systemEnvironment)
+const QStringList TWApp::getBinaryPaths()
 {
-#if defined(Q_OS_WIN)
-#define PATH_CASE_SENSITIVE	Qt::CaseInsensitive
+#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
+	constexpr auto SkipEmptyParts = QString::SkipEmptyParts;
 #else
-#define PATH_CASE_SENSITIVE	Qt::CaseSensitive
+	constexpr auto SkipEmptyParts = Qt::SkipEmptyParts;
 #endif
+
 	QStringList binPaths = getPrefsBinaryPaths();
-	QMutableStringListIterator envIter(systemEnvironment);
-	while (envIter.hasNext()) {
-		QString& envVar = envIter.next();
-		if (envVar.startsWith(QLatin1String("PATH="), PATH_CASE_SENSITIVE)) {
-			foreach (const QString& s, envVar.mid(5).split(QString::fromLatin1(PATH_LIST_SEP), QString::SkipEmptyParts)) {
-				if (!binPaths.contains(s)) {
-					binPaths.append(s);
-				}
-			}
-			envVar = envVar.left(5) + binPaths.join(QString::fromLatin1(PATH_LIST_SEP));
-			break;
+	QProcessEnvironment env{QProcessEnvironment::systemEnvironment()};
+	for(QString & path : binPaths) {
+		path = replaceEnvironmentVariables(path);
+	}
+	for (QString path : env.value(QStringLiteral("PATH")).split(QStringLiteral(PATH_LIST_SEP), SkipEmptyParts)) {
+		path = replaceEnvironmentVariables(path);
+		if (!binPaths.contains(path)) {
+			binPaths.append(path);
 		}
 	}
 	return binPaths;
@@ -534,21 +602,18 @@ void TWApp::writeToMailingList()
 	QString address(QLatin1String("texworks@tug.org"));
 	QString body(QLatin1String("Thank you for taking the time to write an email to the TeXworks mailing list. Please read the instructions below carefully as following them will greatly facilitate the communication.\n\nInstructions:\n-) Please write your message in English (it's in your own best interest; otherwise, many people will not be able to understand it and therefore will not answer).\n\n-) Please type something meaningful in the subject line.\n\n-) If you are having a problem, please describe it step-by-step in detail.\n\n-) After reading, please delete these instructions (up to the \"configuration info\" below which we may need to find the source of problems).\n\n\n\n----- configuration info -----\n"));
 
+	body += QStringLiteral("TeXworks version : %1 (%2) [r.%3, %4]\n").arg(Tw::Utils::VersionInfo::versionString(), Tw::Utils::VersionInfo::buildIdString(), Tw::Utils::VersionInfo::gitCommitHash(), QLocale::system().toString(Tw::Utils::VersionInfo::gitCommitDate().toLocalTime(), QLocale::ShortFormat));
 #if defined(MIKTEX)
-	body += QStringLiteral("TeXworks version : " TEXWORKS_VERSION "r.") + TWUtils::gitCommitHash() + QChar::fromLatin1('\n');
-	body += QStringLiteral("MiKTeX version   : ") + QString::fromUtf8(MiKTeX::Core::Utils::GetMiKTeXBannerString().c_str()) + QChar::fromLatin1('\n');
-#else
-	body += QLatin1String("TeXworks version : " TEXWORKS_VERSION "r.") + TWUtils::gitCommitHash() + QLatin1String(" (" TW_BUILD_ID_STR ")\n");
+	body += QStringLiteral("MiKTeX version   : %1\n").arg(QString::fromUtf8(MiKTeX::Core::Utils::GetMiKTeXBannerString().c_str()));
 #endif
 #if defined(Q_OS_DARWIN)
 	body += QLatin1String("Install location : ") + QDir(applicationDirPath() + QLatin1String("/../..")).absolutePath() + QChar::fromLatin1('\n');
 #else
 	body += QLatin1String("Install location : ") + applicationFilePath() + QChar::fromLatin1('\n');
 #endif
-	body += QLatin1String("Library path     : ") + TWUtils::getLibraryPath(QString()) + QChar::fromLatin1('\n');
+	body += QLatin1String("Library path     : ") + Tw::Utils::ResourcesLibrary::getLibraryPath(QString()) + QChar::fromLatin1('\n');
 
-	QStringList sysEnv(QProcess::systemEnvironment());
-	const QStringList binPaths = getBinaryPaths(sysEnv);
+	const QStringList binPaths = getBinaryPaths();
 	QString pdftex = findProgram(QString::fromLatin1("pdftex"), binPaths);
 	if (pdftex.isEmpty())
 		pdftex = QLatin1String("not found");
@@ -556,9 +621,9 @@ void TWApp::writeToMailingList()
 		QFileInfo info(pdftex);
 		pdftex = info.canonicalFilePath();
 	}
-	
+
 	body += QLatin1String("pdfTeX location  : ") + pdftex + QChar::fromLatin1('\n');
-	
+
 	body += QLatin1String("Operating system : ");
 #if defined(MIKTEX)
         body += QString::fromUtf8(MiKTeX::Core::Utils::GetOSVersionString().c_str()) + QStringLiteral("\n");
@@ -567,22 +632,18 @@ void TWApp::writeToMailingList()
 	body += QLatin1String("Windows ") + GetWindowsVersionString() + QChar::fromLatin1('\n');
 #else
 #if defined(Q_OS_DARWIN)
-#define UNAME_CMDLINE "uname -v"
+	QStringList unameArgs{QStringLiteral("-v")};
 #else
-#define UNAME_CMDLINE "uname -a"
+	QStringList unameArgs{QStringLiteral("-a")};
 #endif
 	QString unameResult(QLatin1String("unknown"));
-	TWSystemCmd unameCmd(this, true);
+	Tw::Utils::SystemCommand unameCmd(this, true);
 	unameCmd.setProcessChannelMode(QProcess::MergedChannels);
-	unameCmd.start(QString::fromLatin1(UNAME_CMDLINE));
+	unameCmd.start(QStringLiteral("uname"), unameArgs);
 	if (unameCmd.waitForStarted(1000) && unameCmd.waitForFinished(1000))
 		unameResult = unameCmd.getResult().trimmed();
 #if defined(Q_OS_DARWIN)
-	SInt32 major = 0, minor = 0, bugfix = 0;
-	Gestalt(gestaltSystemVersionMajor, &major);
-	Gestalt(gestaltSystemVersionMinor, &minor);
-	Gestalt(gestaltSystemVersionBugFix, &bugfix);
-	body += QString::fromLatin1("Mac OS X %1.%2.%3").arg(major).arg(minor).arg(bugfix);
+	body += GetMacOSVersionString();
 	body += QLatin1String(" (") + unameResult + QLatin1String(")\n");
 #else
 	body += unameResult + QChar::fromLatin1('\n');
@@ -711,10 +772,10 @@ QString TWApp::getOpenFileName(QString selectedFilter)
 	                                    filters.join(QLatin1String(";;")), &selectedFilter, options);
 }
 
-void TWApp::open()
+void TWApp::open(const QString & defaultFilter /* = {} */)
 {
 	Tw::Settings settings;
-	QStringList files = getOpenFileNames();
+	QStringList files = getOpenFileNames(defaultFilter);
 	foreach (QString fileName, files) {
 		if (!fileName.isEmpty()) {
 			QFileInfo info(fileName);
@@ -726,7 +787,7 @@ void TWApp::open()
 
 QObject* TWApp::openFile(const QString &fileName, int pos /* = 0 */)
 {
-	if (TWUtils::isPDFfile(fileName)) {
+	if (Tw::Document::isPDFfile(fileName)) {
 		PDFDocumentWindow *doc = PDFDocumentWindow::findDocument(fileName);
 		if (!doc)
 			doc = new PDFDocumentWindow(fileName);
@@ -798,17 +859,20 @@ void TWApp::tileWindows()
 
 void TWApp::arrangeWindows(WindowArrangementFunction func)
 {
-	QDesktopWidget *desktop = QApplication::desktop();
-	for (int screenIndex = 0; screenIndex < desktop->numScreens(); ++screenIndex) {
+	foreach(QScreen * screen, QGuiApplication::screens()) {
 		QWidgetList windows;
-		foreach (TeXDocumentWindow* texDoc, TeXDocumentWindow::documentList())
-			if (desktop->screenNumber(texDoc) == screenIndex)
+		// All windows we iterate over here are top-level windows, so
+		// windowHandle() should return a valid pointer
+		foreach (TeXDocumentWindow* texDoc, TeXDocumentWindow::documentList()) {
+			if (texDoc->windowHandle()->screen() == screen)
 				windows << texDoc;
-		foreach (PDFDocumentWindow* pdfDoc, PDFDocumentWindow::documentList())
-			if (desktop->screenNumber(pdfDoc) == screenIndex)
+		}
+		foreach (PDFDocumentWindow* pdfDoc, PDFDocumentWindow::documentList()) {
+			if (pdfDoc->windowHandle()->screen() == screen)
 				windows << pdfDoc;
+		}
 		if (!windows.empty())
-			(*func)(windows, desktop->availableGeometry(screenIndex));
+			(*func)(windows, screen->availableGeometry());
 	}
 }
 
@@ -830,6 +894,12 @@ bool TWApp::event(QEvent *event)
 
 void TWApp::setDefaultPaths()
 {
+#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
+	constexpr auto SkipEmptyParts = QString::SkipEmptyParts;
+#else
+	constexpr auto SkipEmptyParts = Qt::SkipEmptyParts;
+#endif
+
 	QDir appDir(applicationDirPath());
 	if (!binaryPaths)
 		binaryPaths = new QStringList;
@@ -849,7 +919,7 @@ void TWApp::setDefaultPaths()
 	QString envPath = QString::fromLocal8Bit(getenv("PATH"));
 #endif
 	if (!envPath.isEmpty())
-		foreach (const QString& s, envPath.split(QString::fromLatin1(PATH_LIST_SEP), QString::SkipEmptyParts))
+		foreach (const QString& s, envPath.split(QString::fromLatin1(PATH_LIST_SEP), SkipEmptyParts))
 		if (!binaryPaths->contains(s))
 			binaryPaths->append(s);
 	if (!defaultBinPaths) {
@@ -864,14 +934,20 @@ void TWApp::setDefaultPaths()
             }
           }
 #else
-		foreach (const QString& s, QString::fromUtf8(DEFAULT_BIN_PATHS).split(QString::fromLatin1(PATH_LIST_SEP), QString::SkipEmptyParts)) {
+		foreach (const QString& s, QString::fromUtf8(DEFAULT_BIN_PATHS).split(QString::fromLatin1(PATH_LIST_SEP), SkipEmptyParts)) {
 			if (!binaryPaths->contains(s))
 				binaryPaths->append(s);
 		}
 #endif
 	}
 	for (int i = binaryPaths->count() - 1; i >= 0; --i) {
-		QDir dir(binaryPaths->at(i));
+		// Note: Only replace the environmental variables for testing directory
+		// existance but do not alter the binaryPaths themselves. Those might
+		// get stored, e.g., in the preferences and we want to keep
+		// environmental variables intact in there (as they may be (re)defined
+		// later on).
+		// All binary paths are properly expanded in getBinaryPaths().
+		QDir dir(replaceEnvironmentVariables(binaryPaths->at(i)));
 		if (!dir.exists())
 			binaryPaths->removeAt(i);
 	}
@@ -931,7 +1007,7 @@ void TWApp::setDefaultEngineList()
         defaultEngineIndex = 2;
 #else
 	*engineList
-//		<< Engine("LaTeXmk", "latexmk" EXE, QStringList("-e") << 
+//		<< Engine("LaTeXmk", "latexmk" EXE, QStringList("-e") <<
 //				  "$pdflatex=q/pdflatex -synctex=1 %O %S/" << "-pdf" << "$fullname", true)
 	    << Engine(QString::fromLatin1("pdfTeX"), QString::fromLatin1("pdftex" EXE), QStringList(QString::fromLatin1("$synctexoption")) << QString::fromLatin1("$fullname"), true)
 	    << Engine(QString::fromLatin1("pdfLaTeX"), QString::fromLatin1("pdflatex" EXE), QStringList(QString::fromLatin1("$synctexoption")) << QString::fromLatin1("$fullname"), true)
@@ -975,7 +1051,7 @@ const QList<Engine> TWApp::getEngineList()
 		settings.remove(QString::fromLatin1("engines"));
 
 		if (!foundList) { // read engine list from config file
-			QDir configDir(TWUtils::getLibraryPath(QString::fromLatin1("configuration")));
+			QDir configDir(Tw::Utils::ResourcesLibrary::getLibraryPath(QStringLiteral("configuration")));
 			QFile toolsFile(configDir.filePath(QString::fromLatin1("tools.ini")));
 			if (toolsFile.exists()) {
 				QSettings toolsSettings(toolsFile.fileName(), QSettings::IniFormat);
@@ -1010,7 +1086,7 @@ const QList<Engine> TWApp::getEngineList()
 
 void TWApp::saveEngineList()
 {
-	QDir configDir(TWUtils::getLibraryPath(QString::fromLatin1("configuration")));
+	QDir configDir(Tw::Utils::ResourcesLibrary::getLibraryPath(QStringLiteral("configuration")));
 	QFile toolsFile(configDir.filePath(QString::fromLatin1("tools.ini")));
 	QSettings toolsSettings(toolsFile.fileName(), QSettings::IniFormat);
 	toolsSettings.clear();
@@ -1050,7 +1126,7 @@ const Engine TWApp::getDefaultEngine()
 void TWApp::setDefaultEngine(const QString& name)
 {
 	const QList<Engine> engines = getEngineList();
-	int i;
+	int i{0};
 	for (i = 0; i < engines.count(); ++i) {
 		if (engines[i].name() == name) {
 			Tw::Settings settings;
@@ -1107,6 +1183,29 @@ void TWApp::activatedWindow(QWidget* theWindow)
 	emit hideFloatersExcept(theWindow);
 }
 
+// static
+QStringList TWApp::getTranslationList()
+{
+	QStringList translationList;
+	QVector<QDir> dirs({QDir(QStringLiteral(":/resfiles/translations")), QDir(Tw::Utils::ResourcesLibrary::getLibraryPath(QStringLiteral("translations")))});
+
+	for (QDir transDir : dirs) {
+		for (QFileInfo qmFileInfo : transDir.entryInfoList(QStringList(QStringLiteral(TEXWORKS_NAME "_*.qm")),
+					QDir::Files | QDir::Readable, QDir::Name | QDir::IgnoreCase)) {
+			QString locName = qmFileInfo.completeBaseName();
+			locName.remove(QStringLiteral(TEXWORKS_NAME "_"));
+			if (!translationList.contains(locName, Qt::CaseInsensitive))
+				translationList << locName;
+		}
+	}
+
+	// English is always available, and it has to be the first item
+	translationList.removeAll(QString::fromLatin1("en"));
+	translationList.prepend(QString::fromLatin1("en"));
+
+	return translationList;
+}
+
 void TWApp::applyTranslation(const QString& locale)
 {
 	foreach (QTranslator* t, translators) {
@@ -1128,9 +1227,15 @@ void TWApp::applyTranslation(const QString& locale)
 		names << QString::fromLatin1("qt_") + locale \
 					<< QString::fromLatin1("QtPDF_") + locale \
 					<< QString::fromLatin1(TEXWORKS_NAME) + QString::fromLatin1("_") + locale;
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 		directories << QString::fromLatin1(":/resfiles/translations") \
-								<< QLibraryInfo::location(QLibraryInfo::TranslationsPath) \
-		                        << TWUtils::getLibraryPath(QString::fromLatin1("translations"));
+					<< QLibraryInfo::location(QLibraryInfo::TranslationsPath) \
+					<< Tw::Utils::ResourcesLibrary::getLibraryPath(QStringLiteral("translations"));
+#else
+		directories << QStringLiteral(":/resfiles/translations") \
+					<< QLibraryInfo::path(QLibraryInfo::TranslationsPath) \
+					<< Tw::Utils::ResourcesLibrary::getLibraryPath(QStringLiteral("translations"));
+#endif
 
 		foreach (QString name, names) {
 			foreach (QString dir, directories) {
@@ -1155,7 +1260,7 @@ void TWApp::addToRecentFiles(const QMap<QString,QVariant>& fileProperties)
 	QString fileName = fileProperties.value(QString::fromLatin1("path")).toString();
 	if (fileName.isEmpty())
 		return;
-	
+
 	QList<QVariant> fileList = settings.value(QString::fromLatin1("recentFiles")).toList();
 	QList<QVariant>::iterator i = fileList.begin();
 	while (i != fileList.end()) {
@@ -1216,7 +1321,7 @@ void TWApp::updateScriptsList()
 
 void TWApp::showScriptsFolder()
 {
-	QDesktopServices::openUrl(QUrl::fromLocalFile(TWUtils::getLibraryPath(QString::fromLatin1("scripts"))));
+	QDesktopServices::openUrl(QUrl::fromLocalFile(Tw::Utils::ResourcesLibrary::getLibraryPath(QStringLiteral("scripts"))));
 }
 
 void TWApp::bringToFront()
@@ -1233,7 +1338,7 @@ void TWApp::bringToFront()
 QList<QVariant> TWApp::getOpenWindows() const
 {
 	QList<QVariant> result;
-	
+
 	foreach (QWidget *widget, QApplication::topLevelWidgets()) {
 		if (qobject_cast<TWScriptableWindow*>(widget))
 			result << QVariant::fromValue(qobject_cast<QObject*>(widget));
@@ -1244,15 +1349,19 @@ QList<QVariant> TWApp::getOpenWindows() const
 void TWApp::setGlobal(const QString& key, const QVariant& val)
 {
 	QVariant v = val;
-	
+
 	if (key.isEmpty())
 		return;
-	
+
 	// For objects on the heap make sure we are notified when their lifetimes
 	// end so that we can remove them from our hash accordingly
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 	switch (static_cast<QMetaType::Type>(val.type())) {
+#else
+	switch (val.metaType().id()) {
+#endif
 		case QMetaType::QObjectStar:
-			connect(v.value<QObject*>(), SIGNAL(destroyed(QObject*)), this, SLOT(globalDestroyed(QObject*)));
+			connect(v.value<QObject*>(), &QObject::destroyed, this, &TWApp::globalDestroyed);
 			break;
 		default: break;
 	}
@@ -1262,9 +1371,13 @@ void TWApp::setGlobal(const QString& key, const QVariant& val)
 void TWApp::globalDestroyed(QObject * obj)
 {
 	QHash<QString, QVariant>::iterator i = m_globals.begin();
-	
+
 	while (i != m_globals.end()) {
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 		switch (static_cast<QMetaType::Type>(i.value().type())) {
+#else
+		switch (i.value().metaType().id()) {
+#endif
 			case QMetaType::QObjectStar:
 				if (i.value().value<QObject*>() == obj)
 					i = m_globals.erase(i);
@@ -1281,7 +1394,7 @@ void TWApp::globalDestroyed(QObject * obj)
 /*Q_INVOKABLE static*/
 int TWApp::getVersion()
 {
-	return (VER_MAJOR << 16) | (VER_MINOR << 8) | VER_BUGFIX;
+	return Tw::Utils::VersionInfo::getVersion();
 }
 
 //Q_INVOKABLE
@@ -1304,17 +1417,17 @@ QMap<QString, QVariant> TWApp::openFileFromScript(const QString& fileName, QObje
 		Tw::Scripting::Script * script = qobject_cast<Tw::Scripting::Script*>(scriptApi->GetScript());
 		if (!script)
 			return retVal; // this should never happen
-	
+
 		// relative paths are taken to be relative to the folder containing the
 		// executing script's file
 		QDir scriptDir(QFileInfo(script->getFilename()).dir());
 		QString path = scriptDir.absoluteFilePath(fileName);
-	
+
 		if (!scriptApi->mayReadFile(path, scriptApi->GetTarget())) {
 			// Possibly ask user to override the permissions
 			if (!askUser)
 				return retVal;
-			if (QMessageBox::warning(qobject_cast<QWidget*>(scriptApi->GetTarget()), 
+			if (QMessageBox::warning(qobject_cast<QWidget*>(scriptApi->GetTarget()),
 				tr("Permission request"),
 				tr("The script \"%1\" is trying to open the file \"%2\" without sufficient permissions. Do you want to open the file?")\
 					.arg(script->getTitle(), path),
@@ -1346,12 +1459,12 @@ void TWApp::reloadSpellchecker()
 			texDoc->setSpellcheckLanguage(QString());
 		}
 	}
-	
+
 	// reset dictionaries (getDictionaryList(true) automatically updates all
 	// spell checker menus)
 	Tw::Document::SpellChecker::clearDictionaries();
 	Tw::Document::SpellChecker::getDictionaryList(true);
-	
+
 	// reenable spell checker
 	for (QHash<TeXDocumentWindow*, QString>::iterator it = oldLangs.begin(); it != oldLangs.end(); ++it) {
 		it.key()->setSpellcheckLanguage(it.value());
