@@ -1,6 +1,6 @@
 /* mainwindow.h:                                        -*- C++ -*-
 
-   Copyright (C) 2017-2020 Christian Schenk
+   Copyright (C) 2017-2021 Christian Schenk
 
    This file is part of MiKTeX Console.
 
@@ -42,6 +42,7 @@ class PackageProxyModel;
 class PackageTableModel;
 class RepositoryTableModel;
 class RootTableModel;
+class UILanguageTableModel;
 class UpdateTableModel;
 
 namespace Ui
@@ -49,6 +50,8 @@ namespace Ui
   class MainWindow;
 }
 
+constexpr auto TheNameOfTheGame = "MiKTeX Console";
+
 class MainWindow :
   public QMainWindow
 {
@@ -114,7 +117,15 @@ private:
 private:
   void FindIssues()
   {
-    issues = isSetupMode || dontFindIssues ? std::vector<MiKTeX::Setup::Issue>() : MiKTeX::Setup::SetupService::FindIssues(true, false);
+    if (isSetupMode || dontFindIssues)
+    {
+      issues = std::vector<MiKTeX::Setup::Issue>();
+    }
+    else
+    {
+      auto setupService = MiKTeX::Setup::SetupService::Create();
+      issues = setupService->FindIssues(true, false);
+    }
     checkedIssues = true;
   }
 
@@ -351,6 +362,18 @@ private:
 private slots:
   void OnContextMenuUpdates(const QPoint& pos);
 
+private:
+  void SetupUiUserInterface();
+
+private:
+  void UpdateUiUserInterface();
+
+private slots:
+  void OnUILanguageSelected(int index);
+
+private:
+  UILanguageTableModel* uiLanguageModel = nullptr;
+
 private:
   void SetupUiPackageInstallation();
 
@@ -558,7 +581,7 @@ private:
   void CreateReport();
 
 private:
-  MiKTeX::Core::PathName GetReportFileName();
+  MiKTeX::Util::PathName GetReportFileName();
 
 private:
   void SetupUiCleanup();
@@ -636,7 +659,7 @@ private:
   void Exit();
 
 private:
-  void Restart();
+  void Restart(bool silent = false);
 
 private slots:
   void UnloadFileNameDatabase();
@@ -1052,13 +1075,13 @@ private:
   Q_OBJECT;
 
 public:
-  ChangeLinkTargetDirectoryWorker(const MiKTeX::Core::PathName& linkTargetDirectory) :
+  ChangeLinkTargetDirectoryWorker(const MiKTeX::Util::PathName& linkTargetDirectory) :
     linkTargetDirectory(linkTargetDirectory)
   {
   }
 
 private:
-  MiKTeX::Core::PathName linkTargetDirectory;
+  MiKTeX::Util::PathName linkTargetDirectory;
 
 protected:
   bool Run() override;
