@@ -1,6 +1,6 @@
 /* Setup.cpp:
 
-   Copyright (C) 1999-2020 Christian Schenk
+   Copyright (C) 1999-2021 Christian Schenk
 
    This file is part of the MiKTeX Setup Wizard.
 
@@ -800,12 +800,6 @@ BOOL SetupApp::InitInstance()
     return FALSE;
   }
 
-  if (FAILED(CoInitialize(nullptr)))
-  {
-    AfxMessageBox(T_(_T("The application could not be initialized (2).")), MB_ICONSTOP | MB_OK);
-    return FALSE;
-  }
-
   AfxInitRichEdit2();
 
   try
@@ -1030,8 +1024,6 @@ BOOL SetupApp::InitInstance()
     ReportError(e);
   }
 
-  CoUninitialize();
-
   return FALSE;
 }
 
